package cn.com.qmth.examcloud.exchange.inner.api.provider;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;


import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.exchange.inner.api.SendSmsCloudService;
import cn.com.qmth.examcloud.exchange.inner.api.request.CheckSmsCodeReq;
import cn.com.qmth.examcloud.exchange.inner.api.request.SendSmsReq;
import cn.com.qmth.examcloud.exchange.inner.api.response.CheckIdentifyingCodeResp;
import cn.com.qmth.examcloud.exchange.inner.api.response.SendSmsResp;
import cn.com.qmth.examcloud.exchange.inner.service.SendSmsService;
import cn.com.qmth.examcloud.exchange.inner.service.bean.SendSmsInfo;

/**
 * 
 * @author chenken
 * @date 2018年7月3日 下午4:03:59
 * @company QMTH
 * @description SendSmsProvider.java
 */
@RestController
@RequestMapping("${$rmp.cloud.exchange.inner}" + "/sendSms")
public class SendSmsProvider extends ControllerSupport implements SendSmsCloudService {

	private static final long serialVersionUID = -1497756895732370672L;
	
	@Autowired
	private SendSmsService sendSmsService;

	@Override
	@RequestMapping(method = RequestMethod.POST, value = "/sendIdentifyingCode")
	public SendSmsResp sendIdentifyingCode(@RequestBody SendSmsReq sendSmsReq){
		checkSendSmsReq(sendSmsReq);
		SendSmsInfo sendSmsInfo = new SendSmsInfo();
		sendSmsInfo.setPhone(sendSmsReq.getPhone());
		sendSmsInfo.setCode(sendSmsReq.getCode());
		sendSmsInfo.setSign(sendSmsReq.getSign());
		sendSmsInfo.setTemplateCode(sendSmsReq.getTemplateCode());
		sendSmsInfo.setEffectiveTime(sendSmsReq.getEffectiveTime());
		sendSmsInfo.setIntervalSeconds(sendSmsReq.getIntervalSeconds());
		SendSmsResp sendSmsResp = new SendSmsResp();
		try{
			sendSmsService.sendIdentifyingCode(sendSmsInfo);
			sendSmsResp.setSuccess(true);
			sendSmsResp.setReturnMsg("发送成功");
		}catch(StatusException exception){
			exception.printStackTrace();
			sendSmsResp.setSuccess(false);
			sendSmsResp.setReturnMsg(exception.getDesc());
		}
		return sendSmsResp;
	}
	
	@Override
	@RequestMapping(method = RequestMethod.POST, value = "/checkIdentifyingCode")
	public CheckIdentifyingCodeResp checkIdentifyingCode(@RequestBody CheckSmsCodeReq checkSmsCodeReq){
		if(StringUtils.isBlank(checkSmsCodeReq.getPhone())){
			throw new StatusException("EX-CheckIdentifyingCodeResp-001", "手机号码不能为空");
		}
		if(StringUtils.isBlank(checkSmsCodeReq.getCode())){
			throw new StatusException("EX-CheckIdentifyingCodeResp-002", "验证码不能为空");
		}
		boolean success = sendSmsService.checkIdentifyingCode(checkSmsCodeReq.getPhone(), checkSmsCodeReq.getCode());
		CheckIdentifyingCodeResp resp = new CheckIdentifyingCodeResp();
		resp.setSuccess(success);
		resp.setReturnMsg(success?"校验通过":"校验不通过");
		return resp;
	}

	private void checkSendSmsReq(SendSmsReq sendSmsReq){
		if(StringUtils.isBlank(sendSmsReq.getPhone())){
			throw new StatusException("EX-SendSmsProvider-001", "手机号码不能为空");
		}
		if(StringUtils.isBlank(sendSmsReq.getCode())){
			throw new StatusException("EX-SendSmsProvider-002", "短信验证码不能为空");
		}
		if(StringUtils.isBlank(sendSmsReq.getSign())){
			throw new StatusException("EX-SendSmsProvider-003", "短信签名不能为空");
		}
		if(StringUtils.isBlank(sendSmsReq.getTemplateCode())){
			throw new StatusException("EX-SendSmsProvider-004", "短信模板code不能为空");
		}
	}
	
}
