package cn.com.qmth.examcloud.exchange.outer.api.provider.cug;

import java.io.File;
import java.io.IOException;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.HttpClientUtil;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.commons.web.support.StatusResponse;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.request.SaveStudentReq;
import cn.com.qmth.examcloud.core.basic.api.response.SaveStudentResp;
import cn.com.qmth.examcloud.examwork.api.ExamCloudService;
import cn.com.qmth.examcloud.examwork.api.ExamStudentCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamStudentBean;
import cn.com.qmth.examcloud.examwork.api.request.GetExamReq;
import cn.com.qmth.examcloud.examwork.api.request.SaveExamStudentReq;
import cn.com.qmth.examcloud.examwork.api.response.GetExamResp;
import cn.com.qmth.examcloud.examwork.api.response.SaveExamStudentResp;
import cn.com.qmth.examcloud.exchange.outer.api.CugOuterService;
import cn.com.qmth.examcloud.exchange.outer.api.request.cug.OuterSaveCugStudentAndExamStudentReq;
import cn.com.qmth.examcloud.exchange.outer.api.response.cug.SaveCugStudentAndExamStudentResp;
import cn.com.qmth.examcloud.exchange.outer.service.CourseGroupService;
import cn.com.qmth.examcloud.exchange.outer.service.FaceService;
import cn.com.qmth.examcloud.exchange.outer.service.bean.Course;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;

/**
 * 中国地质大学（武汉）
 *
 * @author WANGWEI
 * @date 2018年9月4日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Api("废弃接口")
@RestController
@RequestMapping("${$rmp.cloud.exchange.outer}/cug")
public class CugOuterServiceProvider extends ControllerSupport implements CugOuterService {

	private static final long serialVersionUID = -2082287868965834214L;

	@Autowired
	StudentCloudService studentCloudService;

	@Autowired
	private FaceService faceService;

	@Autowired
	CourseGroupService courseGroupService;

	@Autowired
	ExamCloudService examCloudService;

	@Autowired
	ExamStudentCloudService examStudentCloudService;

	@Value("${$tempDir}")
	private String tempDir;

	@ApiOperation(value = "废弃接口,考生接入", httpMethod = "POST")
	@ApiResponses({
			@ApiResponse(code = 200, message = "成功", response = SaveCugStudentAndExamStudentResp.class),
			@ApiResponse(code = 500, message = "系统异常(异常信息见响应体)", response = StatusResponse.class)})
	@PostMapping("saveCugStudentAndExamStudent")
	@Override
	public SaveCugStudentAndExamStudentResp saveCugStudentAndExamStudent(
			@RequestBody @ApiParam(required = true) OuterSaveCugStudentAndExamStudentReq req) {

		trim(req);

		Long rootOrgId = req.getRootOrgId();
		String identityNumber = req.getIdentityNumber();

		String courseGroupName = req.getCourseGroupName();
		String examName = req.getExamName();
		String specialtyName = req.getSpecialtyName();

		List<Course> courseList = courseGroupService.getCourseList(rootOrgId, courseGroupName);

		SaveStudentReq request = new SaveStudentReq();
		request.setIdentityNumber(identityNumber);
		request.setName(req.getName());
		request.setOrgCode(req.getOrgCode());
		request.setOrgName(req.getOrgName());
		request.setPhoneNumber(req.getPhoneNumber());
		request.setRemark(req.getRemark());
		request.setRootOrgId(req.getRootOrgId());
		// request.setStudentCode(req.getStudentCode());

		SaveStudentResp response = studentCloudService.saveStudent(request);

		Long studentId = response.getStudentId();

		String photoUrl = req.getPhotoUrl();
		if (StringUtils.isNotBlank(photoUrl)) {
			byte[] bs;
			try {
				bs = HttpClientUtil.get(photoUrl);
			} catch (Exception e) {
				throw new StatusException("EX-100002", "photoUrl is wrong");
			}

			int lastIndexOf = photoUrl.lastIndexOf(".");
			if (0 > lastIndexOf) {
				throw new StatusException("EX-100002",
						"photoPath is not end with photo file suffix.");
			}
			String fileSuffix = photoUrl.substring(lastIndexOf);

			String fileName = identityNumber + fileSuffix;
			File temp = new File(
					tempDir + File.separator + "student_photo" + File.separator + fileName);

			try {
				FileUtils.writeByteArrayToFile(temp, bs);
			} catch (IOException e) {
				throw new StatusException("EX-100003", "文件读写失败");
			}

			faceService.processFace(rootOrgId, identityNumber, fileSuffix, temp, req.getOperator());
		}

		GetExamReq getExamReq = new GetExamReq();
		getExamReq.setName(examName);
		getExamReq.setRootOrgId(rootOrgId);
		GetExamResp getExamResp = examCloudService.getExam(getExamReq);

		Long examId = getExamResp.getId();

		List<Long> examStudentIdList = Lists.newArrayList();

		for (Course course : courseList) {
			SaveExamStudentReq sReq = new SaveExamStudentReq();
			sReq.setCourseLevel(course.getLevel());
			sReq.setCourseCode(course.getCode());
			sReq.setCourseName(course.getName());

			sReq.setExamId(examId);
			sReq.setExamName(examName);

			sReq.setStudentId(studentId);
			sReq.setIdentityNumber(req.getIdentityNumber());
			sReq.setRootOrgId(rootOrgId);
			sReq.setStudentCode(req.getStudentCode());
			sReq.setStudentName(req.getName());
			sReq.setSpecialtyName(specialtyName);

			sReq.setRemark(req.getRemark());
			sReq.setPaperType(req.getPaperType());

			SaveExamStudentResp savedExamStudent = examStudentCloudService.saveExamStudent(sReq);
			ExamStudentBean examStudentBean = savedExamStudent.getExamStudentBean();
			examStudentIdList.add(examStudentBean.getId());
		}

		SaveCugStudentAndExamStudentResp resp = new SaveCugStudentAndExamStudentResp();
		resp.setOrgId(response.getOrgId());
		resp.setOrgName(response.getOrgName());
		resp.setRootOrgId(response.getRootOrgId());
		resp.setStudentId(response.getStudentId());
		resp.setExamStudentIdList(examStudentIdList);

		return resp;
	}

}
